"use strict";

var _babelHelpers = require("babel-runtime/helpers")["default"];

// DNS mask allows you to test against local server using a real URL, for
// example, write a test to visit http://example.com that will load a page from
// localhost.

var DNS = require("dns");
var Net = require("net");

// DNSMask hijacks Node's DNS.resolve and DNS.lookup and returns results based on
// your domain/type -> IP mapping.
module.exports = (function () {
  function DNSMask() {
    _babelHelpers.classCallCheck(this, DNSMask);

    this._domains = {};
    this._lookup = DNS.lookup;
    DNS.lookup = this.lookup.bind(this);
    this._resolve = DNS.resolve;
    DNS.resolve = this.resolve.bind(this);
    this._resolveMx = DNS.resolve;
    DNS.resolveMx = this.resolveMx.bind(this);
  }

  // Requests for the given domain will return 127.0.0.1 (or ::1 for IPv6).
  //
  // Use asterisks to map all subdomains, e.g. *.example.com.

  DNSMask.prototype.localhost = function localhost(domain) {
    this.map(domain, "A", "127.0.0.1");
    this.map(domain, "AAAA", "::1");
  };

  // Requests for the given domain and type will return the specified address.
  //
  // For example:
  //   map("example.com", "CNAME", "localhost")
  //   map("example.com", "A", "127.0.0.1")
  //   map("example.com", "AAAA", "::1")
  //
  // The record type (A, CNAME, etc) must be uppercase.  You can also call with
  // two arguments, to set A record (IPv4) or AAAA (IPv6).  This is equivalent:
  //   map("example.com", "localhost") // CNAME
  //   map("example.com", "127.0.0.1") // A
  //   map("example.com", "::1")       // AAAA
  //
  // Use asterisks to map all subdomains, e.g. *.example.com.

  DNSMask.prototype.map = function map(domain, type, address) {
    if (arguments.length === 2) {
      address = type;
      switch (Net.isIP(address)) {
        case 4:
          {
            type = "A";
            break;
          }
        case 6:
          {
            type = "AAAA";
            break;
          }
        default:
          {
            type = "CNAME";
          }
      }
    }

    if (address) {
      if (!this._domains[domain]) this._domains[domain] = {};
      this._domains[domain][type] = address;
    } else this.unmap(domain, type);
  };

  // Remove all mapping for the given domain/type.  With one argument, removes
  // all mapping for the given domain, of any type.

  DNSMask.prototype.unmap = function unmap(domain, type) {
    if (type) {
      if (this._domains[domain]) delete this._domains[domain][type];
    } else delete this._domains[domain];
  };

  // Alternative implementation for Node's DNS.lookup.

  DNSMask.prototype.lookup = function lookup(domain, family, callback) {
    var _this = this;

    // With two arguments, second argument is the callback, family is 4 or 6
    if (arguments.length === 2) {
      var _ref = [null, family];

      var _ref2 = _babelHelpers.slicedToArray(_ref, 2);

      family = _ref2[0];
      callback = _ref2[1];
    }

    // If domain is missing, lookup returns null IP
    if (!domain) {
      setImmediate(function () {
        callback(null, null, 4);
      });
      return;
    }

    // If lookup called with IP address, resolve that address.
    if (Net.isIP(domain)) {
      setImmediate(function () {
        callback(null, domain, Net.isIP(domain));
      });
      return;
    }

    // First try to resolve CNAME into another domain name, then resolve that to
    // A/AAAA record
    var cname = this._find(domain, "CNAME");
    if (cname) domain = cname;
    if (family === 4 || !family) {
      var _ret = (function () {
        var ipv4 = _this._find(domain, "A");
        if (ipv4) {
          setImmediate(function () {
            callback(null, ipv4, 4);
          });
          return {
            v: undefined
          };
        }
      })();

      if (typeof _ret === "object") {
        return _ret.v;
      }
    }
    if (family === 6 || !family) {
      var _ret2 = (function () {
        var ipv6 = _this._find(domain, "AAAA");
        if (ipv6) {
          setImmediate(function () {
            callback(null, ipv6, 6);
          });
          return {
            v: undefined
          };
        }
      })();

      if (typeof _ret2 === "object") {
        return _ret2.v;
      }
    }
    this._lookup(domain, family, callback);
  };

  // Alternative implementation for Node's DNS.resolve.

  DNSMask.prototype.resolve = function resolve(domain, type, callback) {
    // With two arguments, second argument is the callback, type is 'A'
    if (arguments.length === 2) {
      var _ref = ["A", type];

      var _ref2 = _babelHelpers.slicedToArray(_ref, 2);

      type = _ref2[0];
      callback = _ref2[1];
    }
    var ip = this._find(domain, type);
    if (ip) {
      setImmediate(function () {
        callback(null, [ip]);
      });
    } else this._resolve(domain, type, callback);
  };

  // Alternative implementation for Node's DNS.resolveMx.

  DNSMask.prototype.resolveMx = function resolveMx(domain, callback) {
    var exchange = this._find(domain, "MX");
    if (exchange) {
      setImmediate(function () {
        callback(null, [exchange]);
      });
    } else this._resolveMx(domain, callback);
  };

  // Returns IP address for the given domain/type.

  DNSMask.prototype._find = function _find(domain, type) {
    var _this = this;

    // Turn domain into a list of matches, from most to least specific, e.g.
    // 'foo.example.com' turns into:
    //
    // [ 'foo.example.test',
    //   '*.foo.example.test',
    //   '*.example.test',
    //   '*.test' ]
    var parts = domain.split(".");
    var domains = [domain];
    while (parts.length) {
      domains.push("*." + parts.join("."));
      parts.shift();
    }

    return domains.map(function (pattern) {
      return _this._domains[pattern];
    }).map(function (domain) {
      return domain && domain[type];
    }).filter(function (ip) {
      return ip;
    })[0];
  };

  return DNSMask;
})();
//# sourceMappingURL=dns_mask.js.map