"use strict";

var _babelHelpers = require("babel-runtime/helpers")["default"];

var _core = require("babel-runtime/core-js")["default"];

// The event loop.
//
// Each browser has an event loop, which processes asynchronous events like
// loading pages and resources, XHR, timeouts and intervals, etc. These are
// procesed in order.
//
// The purpose of the event loop is two fold:
// - To get events processed in the right order for the active window (and only
//   the active window)
// - And to allow the code to wait until all events have been processed
//   (browser.wait, .visit, .pressButton, etc)
//
// The event loop has one interesting method: `wait`.
//
// Each window maintains its own event queue. Its interesting methods are
// `enqueue`, `http`, `dispatch` and the timeout/interval methods.

var _require = require("events");

var EventEmitter = _require.EventEmitter;

var ms = require("ms");

// Wrapper for a timeout (setTimeout)

var Timeout = (function () {

  // queue   - Reference to the event queue
  // fn      - When timer fires, evaluate this function
  // delay   - How long to wait
  // remove  - Call this to discard timer
  //
  // Instance variables add:
  // next    - When is this timer firing next
  // handle  - Node.js timeout handle

  function Timeout(queue, fn, delay, remove) {
    var _this = this;

    _babelHelpers.classCallCheck(this, Timeout);

    this.queue = queue;
    this.fn = fn;
    this.delay = Math.max(delay || 0, 0);
    this.remove = remove;

    // When timeout fires, queue event for processing during a wait.
    var fire = function () {
      _this.queue.enqueue(function () {
        _this.queue.browser.emit("timeout", _this.fn, _this.delay);
        try {
          _this.queue.window._evaluate(_this.fn);
        } catch (error) {
          _this.queue.browser.emit("error", error);
        }
      });
      _this.remove();
    };
    this.handle = _core.global.setTimeout(fire, this.delay);
    this.next = Date.now() + this.delay;
  }

  // clearTimeout

  Timeout.prototype.stop = function stop() {
    _core.global.clearTimeout(this.handle);
    this.remove();
  };

  return Timeout;
})();

// Wapper for an interval (setInterval)

var Interval = (function () {

  // queue     - Reference to the event queue
  // fn        - When timer fires, evaluate this function
  // interval  - Interval between firing
  // remove    - Call this to discard timer
  //
  // Instance variables add:
  // next    - When is this timer firing next
  // handle  - Node.js interval handle

  function Interval(queue, fn, interval, remove) {
    var _this = this;

    _babelHelpers.classCallCheck(this, Interval);

    this.queue = queue;
    this.fn = fn;
    this.interval = Math.max(interval || 0);
    this.remove = remove;

    // When interval fires, queue event for processing during a wait.
    // Don't queue if already processing.
    var pendingEvent = false;
    var fire = function () {
      _this.next = Date.now() + _this.interval;
      if (pendingEvent) return;

      pendingEvent = true;
      _this.queue.enqueue(function () {
        pendingEvent = false;
        _this.queue.browser.emit("interval", _this.fn, _this.interval);
        try {
          _this.queue.window._evaluate(_this.fn);
        } catch (error) {
          _this.queue.browser.emit("error", error);
        }
      });
    };
    this.handle = _core.global.setInterval(fire, this.interval);
    this.next = Date.now() + this.interval;
  }

  // clearTimeout

  Interval.prototype.stop = function stop() {
    _core.global.clearInterval(this.handle);
    this.remove();
  };

  return Interval;
})();

// Each window has an event queue that holds all pending events and manages
// timers.
//
// Each event is a function that gets called when it's the event time to fire.
// Various components push new functions to the queue, the event loop is
// reponsible for fetching the events and executing them.
//
// Timers are resumed when the window becomes active, suspened when the window
// becomes inactive, and execute by queuing events.
//
// HTTP request should use the `http` method, which uses `expecting` to indicate
// an event is expected while the request is in progress (so don't stop event
// loop), and queue the event when the response arrives.

var EventQueue = (function () {

  // Instance variables:
  // browser     - Reference to the browser
  // window      - Reference to the window
  // eventLoop   - Reference to the browser's event loop
  // expecting   - These are holding back the event loop
  // queue       - FIFO queue of functions to call
  // timers      - Sparse array of timers (index is the timer handle)
  // nextTimerHandle - Value of next timer handler

  function EventQueue(window) {
    _babelHelpers.classCallCheck(this, EventQueue);

    this.window = window;
    this.browser = window.browser;
    this.eventLoop = this.browser.eventLoop;
    this.queue = [];
    this.expecting = [];
    this.timers = [];
    this.eventSources = [];
    this.nextTimerHandle = 1;
  }

  // Cleanup when we dispose of the window

  EventQueue.prototype.destroy = function destroy() {
    this.queue = null;
    for (var _iterator = this.timers, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _core.$for.getIterator(_iterator);;) {
      var _ref;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref = _i.value;
      }

      var timer = _ref;

      if (timer) timer.stop();
    }
    this.timers = null;
    for (var _iterator2 = this.expecting, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _core.$for.getIterator(_iterator2);;) {
      var _ref2;

      if (_isArray2) {
        if (_i2 >= _iterator2.length) break;
        _ref2 = _iterator2[_i2++];
      } else {
        _i2 = _iterator2.next();
        if (_i2.done) break;
        _ref2 = _i2.value;
      }

      var expecting = _ref2;

      expecting();
    }this.expecting = null;
    for (var _iterator3 = this.eventSources, _isArray3 = Array.isArray(_iterator3), _i3 = 0, _iterator3 = _isArray3 ? _iterator3 : _core.$for.getIterator(_iterator3);;) {
      var _ref3;

      if (_isArray3) {
        if (_i3 >= _iterator3.length) break;
        _ref3 = _iterator3[_i3++];
      } else {
        _i3 = _iterator3.next();
        if (_i3.done) break;
        _ref3 = _i3.value;
      }

      var eventSource = _ref3;

      if (eventSource) eventSource.close();
    }
    this.eventSources = null;
  };

  // -- Events --

  // Add a function to the event queue, to be executed in order.

  EventQueue.prototype.enqueue = function enqueue(fn) {
    if (!this.queue) throw new Error("This browser has been destroyed");
    if (fn) {
      this.queue.push(fn);
      this.eventLoop.run();
    }
  };

  // Event loop uses this to grab event from top of the queue.

  EventQueue.prototype.dequeue = function dequeue() {
    if (!this.queue) {
      return;
    }var fn = this.queue.shift();
    if (fn) {
      return fn;
    }for (var _iterator = [].concat(_babelHelpers.toConsumableArray(this.window.frames)), _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _core.$for.getIterator(_iterator);;) {
      var _ref;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref = _i.value;
      }

      var frame = _ref;

      var childFn = frame._eventQueue.dequeue();
      if (childFn) {
        return childFn;
      }
    }
    return null;
  };

  // Makes an HTTP request.
  //
  // Parameters are:
  // method   - Method (defaults to GET)
  // url      - URL (string)
  // options  - See below
  // callback - Called with error, or null and response
  //
  // Options:
  //   headers   - Name/value pairs of headers to send in request
  //   params    - Parameters to pass in query string or document body
  //   body      - Request document body
  //   timeout   - Request timeout in milliseconds (0 or null for no timeout)
  //
  // Calls callback with response error or null and response object.

  EventQueue.prototype.http = function http(method, url, options, callback) {
    var _this = this;

    if (!this.queue) {
      return;
    }var done = this.eventLoop.expecting();
    this.expecting.push(done);

    this.browser.resources.request(method, url, options, function (error, response) {
      // We can't cancel pending requests, but we can ignore the response if
      // window already closed
      if (_this.queue) {
        // This will get completion function to execute, e.g. to check a page
        // before meta tag refresh
        _this.enqueue(function () {
          callback(error, response);
        });
      }

      if (_this.expecting) {
        _this.expecting.splice(_this.expecting.indexOf(done), 1);
        done();
      }
    });
  };

  // Fire an error event.

  EventQueue.prototype.onerror = function onerror(error) {
    this.browser.emit("error", error);

    var event = this.window.document.createEvent("Event");
    event.initEvent("error", false, false);
    event.message = error.message;
    event.error = error;
    this.window.dispatchEvent(event);
  };

  // -- EventSource --

  EventQueue.prototype.addEventSource = function addEventSource(eventSource) {
    var _this = this;

    if (!this.eventSources) throw new Error("This browser has been destroyed");

    this.eventSources.push(eventSource);

    var emit = eventSource.emit;
    eventSource.emit = function () {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      _this.enqueue(function () {
        emit.apply(eventSource, args);
      });
      _this.eventLoop.emit("server");
    };
  };

  // -- Timers --

  // Window.setTimeout

  EventQueue.prototype.setTimeout = function setTimeout(fn) {
    var _this = this;

    var delay = arguments[1] === undefined ? 0 : arguments[1];

    if (!this.timers) throw new Error("This browser has been destroyed");
    if (!fn) {
      return;
    }var handle = this.nextTimerHandle;
    ++this.nextTimerHandle;
    var remove = function () {
      delete _this.timers[handle];
    };
    this.timers[handle] = new Timeout(this, fn, delay, remove);
    return handle;
  };

  // Window.clearTimeout

  EventQueue.prototype.clearTimeout = function clearTimeout(handle) {
    if (!this.timers) throw new Error("This browser has been destroyed");
    var timer = this.timers[handle];
    if (timer) timer.stop();
  };

  // Window.setInterval

  EventQueue.prototype.setInterval = function setInterval(fn) {
    var _this = this;

    var interval = arguments[1] === undefined ? 0 : arguments[1];

    if (!this.timers) throw new Error("This browser has been destroyed");
    if (!fn) {
      return;
    }var handle = this.nextTimerHandle;
    ++this.nextTimerHandle;
    var remove = function () {
      delete _this.timers[handle];
    };
    this.timers[handle] = new Interval(this, fn, interval, remove);
    return handle;
  };

  // Window.clearInterval

  EventQueue.prototype.clearInterval = function clearInterval(handle) {
    if (!this.timers) throw new Error("This browser has been destroyed");
    var timer = this.timers[handle];
    if (timer) timer.stop();
  };

  // Returns the timestamp of the next timer event

  EventQueue.prototype.next = function next() {
    var next = Infinity;
    for (var _iterator = this.timers, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _core.$for.getIterator(_iterator);;) {
      var _ref;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref = _i.value;
      }

      var timer = _ref;

      if (timer && timer.next < next) next = timer.next;
    }
    for (var _iterator2 = [].concat(_babelHelpers.toConsumableArray(this.window.frames)), _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _core.$for.getIterator(_iterator2);;) {
      var _ref2;

      if (_isArray2) {
        if (_i2 >= _iterator2.length) break;
        _ref2 = _iterator2[_i2++];
      } else {
        _i2 = _iterator2.next();
        if (_i2.done) break;
        _ref2 = _i2.value;
      }

      var frame = _ref2;

      var frameNext = frame._eventQueue.next();
      if (frameNext < next) next = frameNext;
    }
    return next;
  };

  return EventQueue;
})();

// The browser event loop.
//
// All asynchronous events are processed by this one. The event loop monitors one
// event queue, of the currently active window, and executes its events. Other
// windows are suspended.
//
// Reason to wait for the event loop:
// - One or more events waiting in the queue to be processed
// - One or more timers waiting to fire
// - One or more future events, expected to arrive in the queue
//
// Reasons to stop waiting:
// - No more events in the queue, or expected to arrive
// - No more timers, or all timers are further than our timeout
// - Completion function evaluated to true
//
// The event loop emits the following events (on the browser):
// tick  - Emitted after executing an event; single argument is expected time
//         until next tick event (in ms, zero for "soon")
// done  - Emitted when the event queue is empty (may fire more than once)
// error - Emitted when an error occurs
module.exports = (function (EventEmitter) {

  // Instance variables are:
  // active    - The active window
  // browser   - Reference to the browser
  // expected  - Number of events expected to appear (see `expecting` method)
  // running   - True when inside a run loop
  // waiting   - Counts calls in-progess calls to wait

  function EventLoop(browser) {
    var _this = this;

    _babelHelpers.classCallCheck(this, EventLoop);

    this.browser = browser;
    this.active = null;
    this.expected = 0;
    this.running = false;
    this.waiting = 0;
    // Error in event loop propagates to browser
    this.on("error", function (error) {
      _this.browser.emit("error", error);
    });
  }

  _babelHelpers.inherits(EventLoop, EventEmitter);

  // -- The wait function --

  // Wait until one of these happen:
  // 1. We run out of events to process; callback is called with null and false
  // 2. The completion function evaluates to true; callback is called with null
  //    and false
  // 3. The time duration elapsed; callback is called with null and true
  // 2. An error occurs; callback is called with an error
  //
  // Duration is specifies in milliseconds or string form (e.g. "15s").
  //
  // Completion function is called with the currently active window (may change
  // during page navigation or form submission) and how long until the next
  // event, and returns true to stop waiting, any other value to continue
  // processing events.

  EventLoop.prototype.wait = function wait(waitDuration, completionFunction, callback) {
    // Don't wait longer than duration
    waitDuration = ms(waitDuration.toString()) || this.browser.waitDuration;
    var timeoutOn = Date.now() + waitDuration;
    var eventLoop = this;

    // Someone (us) just started paying attention, start processing events
    ++eventLoop.waiting;
    if (eventLoop.waiting === 1) setImmediate(function () {
      return eventLoop.run();
    });

    var timer = setTimeout(done, waitDuration);

    function ontick(next) {
      if (next >= timeoutOn) {
        // Next event too long in the future, or no events in queue
        // (Infinity), no point in waiting
        done();
        return;
      }

      var activeWindow = eventLoop.active;
      if (completionFunction && activeWindow.document.documentElement) {
        try {
          var waitFor = Math.max(next - Date.now(), 0);
          // Event processed, are we ready to complete?
          var completed = completionFunction(activeWindow, waitFor);
          if (completed) done();
        } catch (error) {
          done(error);
        }
      }
    }
    eventLoop.on("tick", ontick);

    eventLoop.once("idle", done);
    // Don't wait if browser encounters an error (event loop errors also
    // propagate to browser)
    eventLoop.browser.once("error", done);

    function done(error) {
      clearTimeout(timer);
      eventLoop.removeListener("tick", ontick);
      eventLoop.removeListener("idle", done);
      eventLoop.browser.removeListener("error", done);

      --eventLoop.waiting;
      if (eventLoop.waiting === 0) eventLoop.browser.emit("idle");

      callback(error);
    }
  };

  EventLoop.prototype.dump = function dump() {
    var output = arguments[0] === undefined ? process.stdout : arguments[0];

    if (this.waiting) output.write("Event loop: waiting\n");else if (this.running) output.write("Event loop: running\n");else if (this.expected) output.write("Event loop: waiting on " + this.expected + " events\n");else output.write("Event loop: idle\n");
  };

  // -- Event queue management --

  // Creates and returns a new event queue (see EventQueue).

  EventLoop.prototype.createEventQueue = function createEventQueue(window) {
    return new EventQueue(window);
  };

  // Set the active window. Suspends processing events from any other window, and
  // switches to processing events from this window's queue.

  EventLoop.prototype.setActiveWindow = function setActiveWindow(window) {
    if (window === this.active) {
      return;
    }this.active = window;
    if (this.active) ;
    this.run(); // new window, new events
  };

  // Call this method when you know an event is coming, but don't have the event
  // yet. For example, when starting an HTTP request, and the event is for
  // processing the response.
  //
  // This method returns a continuation function that you must call eventually,
  // of the event loop will wait forever.

  EventLoop.prototype.expecting = function expecting() {
    var _this = this;

    ++this.expected;
    var done = function () {
      --_this.expected;
      _this.run(); // may be dead waiting for next event
    };
    return done;
  };

  // Cross-breed between expecting() and process.nextTick.  Executes the function
  // in the next tick, but makes sure waiters block for the function.

  EventLoop.prototype.next = function next(fn) {
    var _this = this;

    ++this.expected;
    setImmediate(function () {
      --_this.expected;
      try {
        fn();
        _this.run();
      } catch (error) {
        _this.emit("error", error);
      }
    });
  };

  // -- Event processing --

  // Grabs next event from the queue, processes it and notifies all listeners.
  // Keeps processing until the queue is empty or all listeners are gone. You
  // only need to bootstrap this when you suspect it's not recursing.

  EventLoop.prototype.run = function run() {
    var _this = this;

    // Are we in the midst of another run loop?
    if (this.running) {
      return;
    } // Is there anybody out there?
    if (this.waiting === 0) {
      return;
    } // Are there any open windows?
    if (!this.active) {
      this.emit("idle");
      return;
    }

    // Give other (Node) events a chance to process
    this.running = true;
    setImmediate(function () {
      _this.running = false;
      if (!_this.active || _this.waiting === 0) {
        _this.emit("idle");
        return;
      }

      try {
        var fn = _this.active._eventQueue.dequeue();
        if (fn) {
          // Process queued function, tick, and on to next event
          try {
            fn();
            _this.emit("tick", 0);
            _this.run();
          } catch (error) {
            _this.emit("error", error);
          }
        } else if (_this.expected > 0) {
          // We're waiting for some events to come along, don't know when,
          // but they'll call run for us
          _this.emit("tick", 0);
        } else {
          // All that's left are timers
          var nextTick = _this.active._eventQueue.next();
          _this.emit("tick", nextTick);
        }
      } catch (error) {
        _this.emit("error", error);
      }
    });
  };

  return EventLoop;
})(EventEmitter);
//# sourceMappingURL=eventloop.js.map